@@ -18,15 +18,22 @@ Sentence has 1 unlinked word:
 3: LEFT-WALL this.p is.v [a] the test.n of bfgiuing[!].g and.j-n xxxvfrg[?].a RIGHT-WALL
 4: LEFT-WALL this.p is.v a [the] test.n of bfgiuing[!].g and.j-n xxxvfrg[?].a RIGHT-WALL
 """
+
 from __future__ import print_function
 import sys
+from sys import stdin
 import re
-import itertools
 import argparse
+import readline
 
 from linkgrammar import (Sentence, ParseOptions, Dictionary,
                          LG_Error, LG_TimerExhausted, Clinkgrammar as clg)
 
+def is_python2():
+    return sys.version_info[:1] == (2,)
+
+get_input = raw_input if is_python2() else input
+
 def nsuffix(q):
     return '' if q == 1 else 's'
 
@@ -38,7 +45,13 @@ class Formatter(argparse.HelpFormatter):
 
 #-----------------------------------------------------------------------------#
 
+is_stdin_atty = sys.stdin.isatty()
+
+PROMPT = "sentence-check: " if is_stdin_atty else ""
 DISPLAY_GUESSES = True   # Display regex and POS guesses
+BATCH_LABELS = '*: '
+
+print ("Version:", clg.linkgrammar_get_version())
 
 args = argparse.ArgumentParser(formatter_class=Formatter)
 args.add_argument('lang', nargs='?', default='en',
@@ -50,6 +63,8 @@ args.add_argument("-p", "--position", action="store_true",
                   help="show word sentence position")
 args.add_argument("-nm", "--no-morphology", dest='morphology', action='store_false',
                   help="do not display morphology")
+args.add_argument("-i", "--interactive", action="store_true",
+                  help="interactive mode after each result")
 
 arg = args.parse_args()
 
@@ -67,11 +82,25 @@ po.max_parse_time = 10   # actual parse timeout may be about twice bigger
 po.spell_guess = True if DISPLAY_GUESSES else False
 po.display_morphology = arg.morphology
 
-print("Enter sentences:")
-# iter(): avoid python2 input buffering
-for sentence_text in iter(sys.stdin.readline, ''):
+while True:
+    try:
+        sentence_text = get_input(PROMPT)
+    except EOFError:
+        print("EOF")
+        exit(0)
+
+    if not is_stdin_atty and sentence_text:
+        if sentence_text[0] == '%':
+            continue
+        if sentence_text[0] == '!': # ignore user-settings for now
+            continue
+        if sentence_text[0] in BATCH_LABELS:
+            sentence_text = sentence_text[1:]
     if sentence_text.strip() == '':
         continue
+    if not is_stdin_atty:
+        print("\n" + sentence_text)
+
     sent = Sentence(str(sentence_text), lgdict, po)
     try:
         linkages = sent.parse()
@@ -86,53 +115,70 @@ for sentence_text in iter(sys.stdin.readline, ''):
         print('Cannot parse the input sentence')
         continue
     null_count = sent.null_count()
+
+    if arg.position:
+        print(' ' * len(PROMPT), end='')
+        for p in range (0, len(sentence_text)):
+            print(p%10, end="")
+        print()
+
     if null_count == 0:
-        print("Sentence parsed OK")
+        print("Sentence parsed OK", end='')
+
+    linkages = list(linkages)
+
+    correction_found = False
+    # search for correction suggestions
+    for l in linkages:
+        for word in l.words():
+            if word.find(r'.#') > 0:
+                correction_found = True
+                break
+        if correction_found:
+            break
+
+    if correction_found:
+        print(" - with correction", end='')
+    if null_count == 0:
+        print(".")
 
     guess_found = False
     if DISPLAY_GUESSES:
-        linkages, check_first = itertools.tee(linkages)
         # Check the first linkage for regexed/unknown words
-        linkage = next(check_first)
-        for word in list(linkage.words()):
+        for word in linkages[0].words():
             # search for something[x]
             if re.search(r'\S+\[[^]]+]', word):
                 guess_found = True
                 break
 
     # Show results with unlinked words or guesses
-    if not arg.position and not guess_found and null_count == 0:
-        continue
+    if arg.position or guess_found or correction_found or null_count != 0:
+        print('Sentence has {} unlinked word{}:'.format(
+            null_count, nsuffix(null_count)))
+        result_no = 0
+        uniqe_parse = {}
+        for linkage in linkages:
+            words = list(linkage.words())
+            if str(words) in uniqe_parse:
+                continue
+            result_no += 1
+            uniqe_parse[str(words)] = True
 
+            if arg.position:
+                words_char = []
+                words_byte = []
+                for wi, w in enumerate(words):
+                    if is_python2():
+                        w = w.decode('utf-8')
+                    words_char.append(w + str((linkage.word_char_start(wi), linkage.word_char_end(wi))))
+                    words_byte.append(w + str((linkage.word_byte_start(wi), linkage.word_byte_end(wi))))
 
-    if arg.position:
-        for p in range (0, len(sentence_text)):
-            print(p%10, end="")
-        print()
+                print(u"{}: {}".format(result_no, ' '.join(words_char)))
+                print(u"{}: {}".format(result_no, ' '.join(words_byte)))
+            else:
+                print("{}: {}".format(result_no, ' '.join(words)))
 
-    print('Sentence has {} unlinked word{}:'.format(
-        null_count, nsuffix(null_count)))
-    result_no = 0
-    uniqe_parse = {}
-    for linkage in linkages:
-        words = list(linkage.words())
-        if str(words) in uniqe_parse:
-            continue
-        result_no += 1
-        uniqe_parse[str(words)] = True
-
-        if arg.position:
-            words_char = []
-            words_byte = []
-            wi = 0
-            for w in words:
-                if sys.version_info < (3, 0):
-                    words[wi] = words[wi].decode('utf-8')
-                words_char.append(words[wi] + str((linkage.word_char_start(wi), linkage.word_char_end(wi))))
-                words_byte.append(words[wi] + str((linkage.word_byte_start(wi), linkage.word_byte_end(wi))))
-                wi += 1
-
-            print(u"{}: {}".format(result_no, ' '.join(words_char)))
-            print(u"{}: {}".format(result_no, ' '.join(words_byte)))
-        else:
-            print("{}: {}".format(result_no, ' '.join(words)))
+    if arg.interactive:
+        print("Interactive session (^D to end):")
+        import code
+        code.interact(local=locals())
