@@ -248,14 +248,15 @@ class CParseOptionsTestCase(unittest.TestCase):
 class DBasicParsingTestCase(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
-        cls.d = Dictionary()
+        cls.d, cls.po = Dictionary(), None
 
     @classmethod
     def tearDownClass(cls):
-        del cls.d
-        del cls.parse_sent
+        del cls.d, cls.po
 
-    def parse_sent(self, text, po=ParseOptions()):
+    def parse_sent(self, text, po=None):
+        if po is None:
+            po = ParseOptions()
         return list(Sentence(text, self.d, po).parse())
 
     def test_that_parse_returns_empty_iterator_on_no_linkage(self):
@@ -284,34 +285,34 @@ class DBasicParsingTestCase(unittest.TestCase):
 
     def test_utf8_encoded_string(self):
         result = self.parse_sent("I love going to the café.")
-        self.assertTrue(1 < len(result))
+        self.assertTrue(len(result) > 1)
         self.assertTrue(isinstance(result[0], Linkage))
         self.assertTrue(isinstance(result[1], Linkage))
 
         # def test_unicode_encoded_string(self):
-        if sys.version_info > (3, 0):
-            result = self.parse_sent(u"I love going to the caf\N{LATIN SMALL LETTER E WITH ACUTE}.")
-        else:
+        if is_python2():
             result = self.parse_sent(u"I love going to the caf\N{LATIN SMALL LETTER E WITH ACUTE}.".encode('utf8'))
-        self.assertTrue(1 < len(result))
+        else:
+            result = self.parse_sent(u"I love going to the caf\N{LATIN SMALL LETTER E WITH ACUTE}.")
+        self.assertTrue(len(result) > 1)
         self.assertTrue(isinstance(result[0], Linkage))
         self.assertTrue(isinstance(result[1], Linkage))
 
         # def test_unknown_word(self):
         result = self.parse_sent("I love going to the qertfdwedadt.")
-        self.assertTrue(1 < len(result))
+        self.assertTrue(len(result) > 1)
         self.assertTrue(isinstance(result[0], Linkage))
         self.assertTrue(isinstance(result[1], Linkage))
 
         # def test_unknown_euro_utf8_word(self):
         result = self.parse_sent("I love going to the qéáéğíóşúüñ.")
-        self.assertTrue(1 < len(result))
+        self.assertTrue(len(result) > 1)
         self.assertTrue(isinstance(result[0], Linkage))
         self.assertTrue(isinstance(result[1], Linkage))
 
         # def test_unknown_cyrillic_utf8_word(self):
         result = self.parse_sent("I love going to the доктором.")
-        self.assertTrue(1 < len(result))
+        self.assertTrue(len(result) > 1)
         self.assertTrue(isinstance(result[0], Linkage))
         self.assertTrue(isinstance(result[1], Linkage))
 
@@ -322,11 +323,27 @@ class DBasicParsingTestCase(unittest.TestCase):
         self.assertEqual([len(l) for l in linkage.links()], [6,2,1,1,3,2,1,1,1])
 
     def test_dictionary_locale_definition(self):
+        if is_python2(): # Locale stuff seems to be broken
+            raise unittest.SkipTest("Test not supported with Python2")
+
+        # python2: Gets system locale (getlocale() is not better)
+        oldlocale = locale.setlocale(locale.LC_CTYPE, None)
+        #print('Current locale:', oldlocale)
+        #print('toupper hij:', 'hij'.upper())
+
         tr_locale = 'tr_TR.UTF-8' if os.name != 'nt' else 'Turkish'
-        oldlocale = locale.setlocale(locale.LC_CTYPE, tr_locale)
+        locale.setlocale(locale.LC_CTYPE, tr_locale)
+        #print('Turkish locale:', locale.setlocale(locale.LC_CTYPE, None))
+
+        # python2: prints HiJ (lowercase small i in the middle)
+        #print('toupper hij:', 'hij'.upper())
+
         self.assertEqual(list(self.parse_sent('Is it fine?')[0].words()),
-             ['LEFT-WALL', 'is.v', 'it', 'fine.a', '?', 'RIGHT-WALL'])
+                         ['LEFT-WALL', 'is.v', 'it', 'fine.a', '?', 'RIGHT-WALL'])
+
         locale.setlocale(locale.LC_CTYPE, oldlocale)
+        #print("Restored locale:", locale.setlocale(locale.LC_CTYPE))
+        #print('toupper hij:', 'hij'.upper())
 
     # If \w is supported, other \ shortcuts are hopefully supported too.
     def test_regex_class_shortcut_support(self):
@@ -340,7 +357,7 @@ class DBasicParsingTestCase(unittest.TestCase):
                           "This should take more than one second to parse! " * 20,
                           ParseOptions(max_parse_time=1))
 
-# The tests here are are numbered since their order is important.
+# The tests here are numbered since their order is important.
 # They depend on the result and state of the previous ones as follows:
 # - set_handler() returned a value that depend on it previous invocation.
 # - A class variable "handler" to record its previous results.
@@ -351,9 +368,11 @@ class EErrorFacilityTestCase(unittest.TestCase):
         "previous": lambda x, y=None: None
     }
 
-    def setUp(self): # pylint: disable=attribute-defined-outside-init,no-member
+    def setUp(self):
         self.testit = "testit"
         self.testleaks = 0  # A repeat count for validating no memory leaks
+        self.numerr = 0
+        self.errinfo = clg.lg_None
 
     @staticmethod
     def error_handler_test(errinfo, data):
@@ -388,9 +407,9 @@ class EErrorFacilityTestCase(unittest.TestCase):
         self.gotit = None
         for i in range(0, 2+self.testleaks):
             self.numerr = LG_Error.printall(self.error_handler_test, self)
-            if 0 == i:
+            if i == 0:
                 self.assertEqual(self.numerr, 1)
-            if 1 == i:
+            if i == 1:
                 self.assertEqual(self.numerr, 0)
         self.assertEqual((self.errinfo.severity, self.errinfo.severity_label), (clg.lg_Error, "Error"))
         self.assertEqual(self.gotit, "testit")
@@ -537,8 +556,7 @@ class HEnglishLinkageTestCase(unittest.TestCase):
 
     @classmethod
     def tearDownClass(cls):
-        del cls.d
-        del cls.po
+        del cls.d, cls.po
 
     def parse_sent(self, text):
         return list(Sentence(text, self.d, self.po).parse())
@@ -712,6 +730,50 @@ class HEnglishLinkageTestCase(unittest.TestCase):
 "\nLEFT-WALL we are.v from the planet.n Gorpon[!]"
 "\n\n")
 
+class GSQLDictTestCase(unittest.TestCase):
+    @classmethod
+    def setUpClass(cls):
+        #clg.parse_options_set_verbosity(clg.parse_options_create(), 3)
+        cls.d, cls.po = Dictionary(lang='demo-sql'), ParseOptions()
+
+    @classmethod
+    def tearDownClass(cls):
+        del cls.d, cls.po
+
+    def test_getting_links(self):
+        linkage_testfile(self, self.d, self.po)
+
+    def test_getting_links_sat(self):
+        sat_po = ParseOptions(use_sat=True)
+        if sat_po.use_sat != True:
+            raise unittest.SkipTest("Library not configured with SAT parser")
+        linkage_testfile(self, self.d, sat_po)
+
+class IWordPositionTestCase(unittest.TestCase):
+    @classmethod
+    def setUpClass(cls):
+        cls.d_en = Dictionary(lang='en')
+
+    @classmethod
+    def tearDownClass(cls):
+        del cls.d_en
+
+    def test_en_word_positions(self):
+        linkage_testfile(self, self.d_en, ParseOptions(), 'pos')
+
+    def test_en_spell_word_positions(self):
+        po = ParseOptions(spell_guess=1)
+        if po.spell_guess == 0:
+            raise unittest.SkipTest("Library is not configured with spell guess")
+        linkage_testfile(self, self.d_en, po, 'pos-spell')
+
+    def test_ru_word_positions(self):
+        linkage_testfile(self, Dictionary(lang='ru'), ParseOptions(), 'pos')
+
+    def test_he_word_positions(self):
+        linkage_testfile(self, Dictionary(lang='he'), ParseOptions(), 'pos')
+
+# Tests are run in alphabetical order; do the language tests last.
 
 class ZENLangTestCase(unittest.TestCase):
     @classmethod
@@ -720,8 +782,7 @@ class ZENLangTestCase(unittest.TestCase):
 
     @classmethod
     def tearDownClass(cls):
-        del cls.d
-        del cls.po
+        del cls.d, cls.po
 
     def test_getting_links(self):
         linkage_testfile(self, self.d, self.po)
@@ -770,10 +831,9 @@ class ZENConstituentsCase(unittest.TestCase):
 
     @classmethod
     def tearDownClass(cls):
-        del cls.d
-        del cls.po
+        del cls.d, cls.po
 
-    def test_a_constiuents_after_parse_list(self):
+    def test_a_constituents_after_parse_list(self):
         """
         Validate that the post-processing data of the first linkage is not
         getting clobbered by later linkages.
@@ -782,7 +842,6 @@ class ZENConstituentsCase(unittest.TestCase):
         self.assertEqual(linkages[0].constituent_tree(),
                 "(S (NP this.p)\n   (VP is.v\n       (NP a test.n))\n   .)\n")
 
-# Tests are run in alphabetical order; do the language tests last.
 class ZDELangTestCase(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
@@ -790,8 +849,7 @@ class ZDELangTestCase(unittest.TestCase):
 
     @classmethod
     def tearDownClass(cls):
-        del cls.d
-        del cls.po
+        del cls.d, cls.po
 
     def parse_sent(self, text):
         return list(Sentence(text, self.d, self.po).parse())
@@ -838,8 +896,7 @@ class ZRULangTestCase(unittest.TestCase):
 
     @classmethod
     def tearDownClass(cls):
-        del cls.d
-        del cls.po
+        del cls.d, cls.po
 
     def parse_sent(self, text):
         return list(Sentence(text, self.d, self.po).parse())
@@ -887,73 +944,111 @@ def linkage_testfile(self, lgdict, popt, desc = ''):
     linkage diagrams / constituent printings.
     """
     self.__class__.longMessage = True
-    if '' != desc:
+    self.maxDiff = None
+    if desc != '':
         desc = desc + '-'
     testfile = clg.test_data_srcdir + "parses-" + desc + clg.dictionary_get_lang(lgdict._obj) + ".txt"
     parses = open(testfile, "rb")
     diagram = None
     constituents = None
+    wordpos = None
     sent = None
     lineno = 0
+    last_opcode = None
+
+    def getwordpos(lkg):
+        words_char = []
+        words_byte = []
+        for wi, w in enumerate(lkg.words()):
+            words_char.append(w + str((linkage.word_char_start(wi), linkage.word_char_end(wi))))
+            words_byte.append(w + str((linkage.word_byte_start(wi), linkage.word_byte_end(wi))))
+        return ' '.join(words_char) + '\n' + ' '.join(words_byte) + '\n'
 
     # Function code and file format sanity check
-    self.opcode_detected = 0
-    def validate_opcode(ctxt=self, O=False, C=False):
-        ctxt.opcode_detected += 1
-        if O:
+    def validate_opcode(opcode):
+        if opcode != ord('O'):
             self.assertFalse(diagram, "at {}:{}: Unfinished diagram entry".format(testfile, lineno))
-        if C:
+        if opcode != ord('C'):
             self.assertFalse(constituents, "at {}:{}: Unfinished constituents entry".format(testfile, lineno))
+        if opcode != ord('P'):
+            self.assertFalse(wordpos, "at {}:{}: Unfinished word-position entry".format(testfile, lineno))
 
     for line in parses:
         lineno += 1
-        if sys.version_info > (3, 0):
+        if not is_python2():
             line = line.decode('utf-8')
+
+        validate_opcode(ord(line[0])) # Use ord() for python2/3 compatibility
+        prev_opcode = last_opcode
+        if line[0] in 'INOCP':
+            last_opcode = line[0]
+
         # Lines starting with I are the input sentences
-        if 'I' == line[0]:
-            validate_opcode(O=True, C=True)
-            sent = line[1:]
+        if line[0] == 'I':
+            sent = line[1:].rstrip('\r\n') # Strip whitespace before RIGHT-WALL (for P)
             diagram = ""
             constituents = ""
+            wordpos = ""
             linkages = Sentence(sent, lgdict, popt).parse()
             linkage = next(linkages, None)
             self.assertTrue(linkage, "at {}:{}: Sentence has no linkages".format(testfile, lineno))
 
         # Generate the next linkage of the last input sentence
-        if 'N' == line[0]:
-            validate_opcode(O=True, C=True)
+        elif line[0] == 'N':
             diagram = ""
             constituents = ""
+            wordpos = ""
             linkage = next(linkages, None)
             self.assertTrue(linkage, "at {}:{}: Sentence has too few linkages".format(testfile, lineno))
 
         # Lines starting with O are the parse diagram
         # It ends with an empty line
-        if 'O' == line[0]:
-            validate_opcode(C=True)
+        elif line[0] == 'O':
             diagram += line[1:]
-            if '\n' == line[1] and 1 < len(diagram):
+            if line[1] == '\n' and len(diagram) > 1:
                 self.assertEqual(linkage.diagram(), diagram, "at {}:{}".format(testfile, lineno))
                 diagram = None
 
         # Lines starting with C are the constituent output (type 1)
         # It ends with an empty line
-        if 'C' == line[0]:
-            validate_opcode(O=True)
-            if '\n' == line[1] and 1 < len(constituents):
+        elif line[0] == 'C':
+            if line[1] == '\n' and len(constituents) > 1:
                 self.assertEqual(linkage.constituent_tree(), constituents, "at {}:{}".format(testfile, lineno))
                 constituents = None
             else:
                 constituents += line[1:]
+
+        # Lines starting with P contain word positions "word(start, end) ... "
+        # The first P line contains character positions
+        # The second P line contains byte positions
+        # It ends with an empty line
+        elif line[0] == 'P':
+            if line[1] == '\n' and len(wordpos) > 1:
+                self.assertEqual(getwordpos(linkage), wordpos, "at {}:{}".format(testfile, lineno))
+                wordpos = None
+            else:
+                wordpos += line[1:]
+
+        # Lines starting with "-" contain a Parse Option
+        elif line[0] == '-':
+            exec('popt.' + line[1:], None, locals())
+
+        elif line[0] in '%\r\n':
+            pass
+        else:
+            self.fail('\nTest file "{}": Invalid opcode "{}" (ord={})'.format(testfile, line[0], ord(line[0])))
+
     parses.close()
 
-    validate_opcode(O=True, C=True)
-    self.assertGreaterEqual(self.opcode_detected, 2, "Nothing has been done for " + testfile)
+    self.assertIn(last_opcode , 'OCP', "Missing result comparison in " + testfile)
 
 def warning(*msg):
     progname = os.path.basename(sys.argv[0])
     print("{}: Warning:".format(progname), *msg, file=sys.stderr)
 
+def is_python2():
+    return sys.version_info[:1] == (2,)
+
 
 import tempfile
 
@@ -983,7 +1078,7 @@ class divert_start(object):
 
     __del__ = divert_end
 
-# Decorate Sentence.parse with eqcost_soretd_parse.
+# Decorate Sentence.parse with eqcost_sorted_parse.
 lg_testutils.add_eqcost_linkage_order(Sentence)
 
 unittest.main()
