@@ -119,7 +119,9 @@ Parse_Options parse_options_create(void)
 	po->max_null_count = 0;
 	po->islands_ok = false;
 	po->use_sat_solver = false;
+#ifdef USE_VITERBI
 	po->use_viterbi = false;
+#endif
 	po->linkage_limit = 100;
 #if defined HAVE_HUNSPELL || defined HAVE_ASPELL
 	po->use_spell_guess = 7;
@@ -292,6 +294,7 @@ bool parse_options_get_use_sat_parser(Parse_Options opts) {
 	return opts->use_sat_solver;
 }
 
+#ifdef USE_VITERBI
 void parse_options_set_use_viterbi(Parse_Options opts, bool dummy) {
 	opts->use_viterbi = dummy;
 }
@@ -299,6 +302,7 @@ void parse_options_set_use_viterbi(Parse_Options opts, bool dummy) {
 bool parse_options_get_use_viterbi(Parse_Options opts) {
 	return opts->use_viterbi;
 }
+#endif
 
 void parse_options_set_linkage_limit(Parse_Options opts, int dummy)
 {
@@ -533,6 +537,9 @@ void sentence_delete(Sentence sent)
 	global_rand_state = sent->rand_state;
 	pool_delete(sent->fm_Match_node);
 	pool_delete(sent->Table_connector_pool);
+	if (IS_DB_DICT(sent->dict))
+		condesc_reuse(sent->dict);
+
 	free(sent);
 }
 
@@ -629,6 +636,11 @@ int sentence_parse(Sentence sent, Parse_Options opts)
 
 	resources_reset(opts->resources);
 
+	/* When the SQL dict is used, expressions are read on demand, so
+	 * the connector descriptor table is not yet ready at this point. */
+	if (IS_DB_DICT(sent->dict))
+		condesc_setup(sent->dict);
+
 	/* Expressions were set up during the tokenize stage.
 	 * Prune them, and then parse.
 	 */
