@@ -1,8 +1,8 @@
 /*
  * cluster.h
  *
- * Data for related-word clusters. Meant to expand disjunct covereage
- * for the case where a parse cannot be completed without ommitting 
+ * Data for related-word clusters. Meant to expand disjunct coverage
+ * for the case where a parse cannot be completed without omitting
  * a word.
  *
  * Copyright (c) 2009 Linas Vepstas <linasvepstas@gmail.com>
@@ -16,6 +16,9 @@
 #include "../api-types.h"
 #include "../link-includes.h"
 
+/* Upper bound on the cost of any connector. */
+#define MAX_CONNECTOR_COST 1000.0
+
 Cluster * lg_cluster_new(void);
 void lg_cluster_delete(Cluster *);
 
