@@ -14,10 +14,10 @@
 #ifndef _LG_DICT_COMMON_H_
 #define  _LG_DICT_COMMON_H_
 
-#include "api-types.h" // for pp_knowledge
-#include "connectors.h" // for condest_t
+#include "api-types.h"                  // pp_knowledge
+#include "connectors.h"                 // ConTable
 #include "dict-structures.h"
-#include "utilities.h" // for locale_t
+#include "utilities.h"                  // locale_t
 
 #define EMPTY_CONNECTOR "ZZZ"
 #define UNLIMITED_CONNECTORS_WORD ("UNLIMITED-CONNECTORS")
@@ -67,6 +67,12 @@ struct Afdict_class_struct
 #define MAX_TOKEN_LENGTH 250     /* Maximum number of chars in a token */
 #define IDIOM_LINK_SZ 5
 
+#ifdef HAVE_SQLITE
+#define IS_DB_DICT(dict) (NULL != dict->db_handle)
+#else
+#define IS_DB_DICT(dict) false
+#endif /* HAVE_SQLITE */
+
 struct Dictionary_s
 {
 	Dict_node *  root;
