@@ -141,7 +141,7 @@ static void dict_error2(Dictionary dict, const char * s, const char *s2)
 	{
 		pos += snprintf(t, ERRBUFLEN, "\"%s\" ", dict->token);
 		strncat(tokens, t, ERRBUFLEN-1-pos);
-		link_advance(dict);
+		if (!link_advance(dict)) break;
 	}
 	tokens[pos] = '\0';
 
@@ -208,6 +208,7 @@ static bool get_character(Dictionary dict, int quote_mode, utf8char uc)
 				}
 			}
 			while ((c != 0x0) && (c != '\n')) c = *(dict->pin++);
+			if (c == 0x0) break;
 			dict->line_number++;
 			continue;
 		}
@@ -234,7 +235,7 @@ static bool get_character(Dictionary dict, int quote_mode, utf8char uc)
 			{
 				dict->pin--;
 				uc[i] = 0x0;
-				return uc;
+				return true;
 			}
 			uc[i] = c;
 			i++;
@@ -325,6 +326,11 @@ static bool link_advance(Dictionary dict)
 				dict_error(dict, "White space inside of token");
 				return false;
 			}
+			if (c[0] == '\0')
+			{
+				dict_error(dict, "EOF while reading quoted token");
+				return false;
+			}
 
 			nr = 0;
 			while (c[nr]) {dict->token[i] = c[nr]; i++; nr++; }
@@ -397,16 +403,16 @@ static bool check_connector(Dictionary dict, const char * s)
 		return false;
 	}
 	if (*s == '@') s++;
-	if (!isupper((int)*s) && ('h' != *s) && ('d' != *s)) {
+	if (!isupper((unsigned char)*s) && ('h' != *s) && ('d' != *s)) {
 		dict_error(dict, "The first letter of a connector must be h,d or uppercase.");
 		return false;
 	}
-	if ((*s == 'I') && (*(s+1) == 'D')) {
+	if ((*s == 'I') && (*(s+1) == 'D') && isupper((unsigned char)*(s+2))) {
 		dict_error(dict, "Connectors beginning with \"ID\" are forbidden");
 		return false;
 	}
 	while (*(s+1)) {
-		if ((!isalnum((int)*s)) && (*s != WILD_TYPE)) {
+		if ((!isalnum((unsigned char)*s)) && (*s != WILD_TYPE)) {
 			dict_error(dict, "All letters of a connector must be ASCII alpha-numeric.");
 			return false;
 		}
@@ -1447,12 +1453,13 @@ Dict_node * insert_dict(Dictionary dict, Dict_node * n, Dict_node * newnode)
 
 	static Exp null_exp = { .type = AND_type, .u.l = NULL };
 	int comp = dict_order_strict(newnode->string, n);
-	if (0 == comp)
+	if (0 == comp &&
+	    /* Suppress reporting duplicate idioms until they are fixed. */
+	    (!contains_underbar(newnode->string) || test_enabled("dup-idioms")))
 	{
 		char t[80+MAX_TOKEN_LENGTH];
 		snprintf(t, sizeof(t),
-		         "Ignoring word \"%s\", which has been multiply defined "
-		         "(use verbosity=2 for more details): ",
+		         "Ignoring word \"%s\", which has been multiply defined:",
 		         newnode->string);
 		dict_error(dict, t);
 		/* Too late to skip insertion - insert it with a null expression. */
@@ -1579,11 +1586,7 @@ void insert_list(Dictionary dict, Dict_node * p, int l)
 	dn_second_half = dn->left;
 	dn->left = dn->right = NULL;
 
-	if (contains_underbar(dn->string))
-	{
-		insert_idiom(dict, dn);
-	}
-	else if (is_idiom_word(dn->string))
+	if (is_idiom_word(dn->string))
 	{
 		prt_error("Warning: Word \"%s\" found near line %d of %s.\n"
 		        "\tWords ending \".Ix\" (x a number) are reserved for idioms.\n"
@@ -1593,6 +1596,11 @@ void insert_list(Dictionary dict, Dict_node * p, int l)
 	}
 	else
 	{
+		if (contains_underbar(dn->string))
+		{
+			insert_idiom(dict, dn);
+		}
+
 		dict->root = insert_dict(dict, dict->root, dn);
 		insert_length_limit(dict, dn);
 		dict->num_entries++;
@@ -1792,7 +1800,8 @@ static bool read_entry(Dictionary dict)
 	/* pass the ; */
 	if (!link_advance(dict))
 	{
-		goto syntax_error;
+		/* Avoid freeing dn, since it is already inserted into the dict. */
+		return false;
 	}
 
 	return true;
