@@ -101,13 +101,17 @@ static inline unsigned int old_hash_disjunct(disjunct_dup_table *dt, Disjunct *
 	unsigned int i;
 	i = 0;
 	for (e = d->left ; e != NULL; e = e->next) {
-		i += e->desc->str_hash;
+		i = (5 * (i + e->desc->uc_num)) + (unsigned int)e->desc->lc_letters + 7;
 	}
 	for (e = d->right ; e != NULL; e = e->next) {
-		i += e->desc->str_hash;
+		i = (5 * (i + e->desc->uc_num)) + (unsigned int)e->desc->lc_letters + 7;
 	}
+#if 0 /* Redundant - the connector hashing has enough entropy. */
 	i += string_hash(d->word_string);
+#endif
 	i += (i>>10);
+
+	d->dup_hash = i;
 	return (i & (dt->dup_table_size-1));
 }
 
@@ -149,6 +153,26 @@ static bool disjuncts_equal(Disjunct * d1, Disjunct * d2)
 	return (strcmp(d1->word_string, d2->word_string) == 0);
 }
 
+#if 0
+int de_fp = 0;
+int de_total = 0;
+static void disjuncts_equal_stat(void)
+{
+		fprintf(stderr, "disjuncts_equal FP %d/%d\n", de_fp, de_total);
+}
+
+static bool disjuncts_equal(Disjunct * d1, Disjunct * d2)
+{
+	if (de_total == 0) atexit(disjuncts_equal_stat);
+	de_total++;
+
+	bool rc = disjuncts_equal1(d1, d2);
+	if (!rc) de_fp++;
+
+	return rc;
+}
+#endif
+
 /**
  * Duplicate the given connector chain.
  * If the argument is NULL, return NULL.
@@ -371,6 +395,7 @@ Disjunct * eliminate_duplicate_disjuncts(Disjunct * d)
 
 		for (dx = dt->dup_table[h]; dx != NULL; dx = dx->next)
 		{
+			if (d->dup_hash != dx->dup_hash) continue;
 			if (disjuncts_equal(dx, d)) break;
 		}
 		if (dx == NULL)
