@@ -209,11 +209,8 @@ static void default_error_handler(lg_errinfo *lge, void *data)
 {
 	FILE *outfile = stdout;
 
-	if (((NULL == data) && (lge->severity <= lg_Debug)) ||
-	    ((NULL != data) && (lge->severity <= *(lg_error_severity *)data) &&
-	     (lg_None !=  lge->severity)))
 	if (((NULL == data) && (lge->severity < lg_Debug)) ||
-	    ((NULL != data) && (lge->severity < *(lg_error_severity *)data) &&
+	    ((NULL != data) && (lge->severity < *(lg_error_severity *)(int *)data) &&
 	     (lg_None !=  lge->severity)))
 	{
 		fflush(stdout); /* Make sure that stdout has been written out first. */
@@ -332,8 +329,8 @@ void err_msgc(err_ctxt *ec, lg_error_severity sev, const char *fmt, ...)
  * See \link verr_msg \endlink for how the severity is handled
  * if it is not specified.
  *
- * @fmt printf()-like format.
- * @... printf()-like arguments.
+ * @param fmt printf()-like format.
+ * @param ... printf()-like arguments.
  * @return Always 0, not to be used. This is needed so prt_error()
  * can be used in complex macros that have to use the comma operator.
  */
@@ -367,6 +364,7 @@ const char *feature_enabled(const char * list, ...)
 
 	while (NULL != (feature = va_arg(given_features, char *)))
 	{
+		if ('\0' == feature[0]) continue;
 		size_t len = strlen(feature);
 		char *buff = alloca(len + 2 + 1); /* leading comma + comma/colon + NUL */
 
@@ -384,9 +382,17 @@ const char *feature_enabled(const char * list, ...)
 		strcpy(buff+1, feature);
 		strcat(buff, ",");
 
-		if (NULL != strstr(list, buff)) return ",";
+		if (NULL != strstr(list, buff))
+		{
+			va_end(given_features);
+			return ",";
+		}
 		buff[len+1] = ':'; /* check for "feature:param" */
-		if (NULL != strstr(list, buff)) return strstr(list, buff) + len + 1;
+		if (NULL != strstr(list, buff))
+		{
+			va_end(given_features);
+			return strstr(list, buff) + len + 1;
+		}
 	}
 	va_end(given_features);
 
