@@ -71,7 +71,7 @@ struct Linkage_s
 	Gword **wg_path_display;      /* Wordgraph path after morpheme combining */
 
 	Linkage_info    lifo;         /* Parse_set index and cost information */
-	PP_info *       pp_info;      /* PP domain info, one for each link */
+	PP_domains *    pp_domains;   /* PP domain info, one for each link */
 
 	Sentence        sent;         /* Used for common linkage data */
 };
