@@ -16,9 +16,11 @@
 #include "disjunct-utils.h"
 #include "fast-match.h"
 #include "string-set.h"
+#include "dict-common/dict-common.h"   // For contable
 #include "tokenize/word-structures.h"  // For Word_struct
 #include "tokenize/wordgraph.h"
 #include "tokenize/tok-structures.h" // XXX TODO provide gword access methods!
+#include "utilities.h"                  // UNREACHABLE
 
 /**
  * The entire goal of this file is provide a fast lookup of all of the
@@ -73,6 +75,7 @@ static void push_match_list_element(fast_matcher_t *ctxt, Disjunct *d)
 	if (ctxt->match_list_end >= ctxt->match_list_size)
 	{
 		ctxt->match_list_size *= MATCH_LIST_SIZE_INC;
+		/* XXX the realloc clobbers xalloc count */
 		ctxt->match_list = realloc(ctxt->match_list,
 		                      ctxt->match_list_size * sizeof(*ctxt->match_list));
 	}
@@ -80,35 +83,16 @@ static void push_match_list_element(fast_matcher_t *ctxt, Disjunct *d)
 	ctxt->match_list[ctxt->match_list_end++] = d;
 }
 
-static void free_match_list(Match_node * t)
-{
-	Match_node *xt;
-	for (; t!=NULL; t=xt) {
-		xt = t->next;
-		xfree((char *)t, sizeof(Match_node));
-	}
-}
-
 /**
  * Free all of the hash tables and Match_nodes
  */
-void free_fast_matcher(fast_matcher_t *mchxt)
+void free_fast_matcher(Sentence sent, fast_matcher_t *mchxt)
 {
-	size_t w;
-	unsigned int i;
-
 	if (NULL == mchxt) return;
-	for (w = 0; w < mchxt->size; w++)
+
+	for (WordIdx w = 0; w < mchxt->size; w++)
 	{
-		for (i = 0; i < mchxt->l_table_size[w]; i++)
-		{
-			free_match_list(mchxt->l_table[w][i]);
-		}
 		xfree((char *)mchxt->l_table[w], mchxt->l_table_size[w] * sizeof (Match_node *));
-		for (i = 0; i < mchxt->r_table_size[w]; i++)
-		{
-			free_match_list(mchxt->r_table[w][i]);
-		}
 		xfree((char *)mchxt->r_table[w], mchxt->r_table_size[w] * sizeof (Match_node *));
 	}
 
@@ -123,7 +107,6 @@ void free_fast_matcher(fast_matcher_t *mchxt)
 
 /**
  * Adds the match node m to the sorted list of match nodes l.
- * The parameter dir determines the order of the sorting to be used.
  * Makes the list sorted from smallest to largest.
  */
 static Match_node * add_to_right_table_list(Match_node * m, Match_node * l)
@@ -156,7 +139,6 @@ static Match_node * add_to_right_table_list(Match_node * m, Match_node * l)
 
 /**
  * Adds the match node m to the sorted list of match nodes l.
- * The parameter dir determines the order of the sorting to be used.
  * Makes the list sorted from largest to smallest
  */
 static Match_node * add_to_left_table_list(Match_node * m, Match_node * l)
@@ -190,26 +172,10 @@ static Match_node * add_to_left_table_list(Match_node * m, Match_node * l)
 /**
  * Compare only the uppercase part of two connectors.
  * Return true if they are the same, else false.
- * FIXME: Use connector enumeration.
  */
 static bool con_uc_eq(const Connector *c1, const Connector *c2)
 {
-	if (string_set_cmp(c1->string, c2->string)) return true;
-	if (c1->hash != c2->hash) return false;
-	if (c1->uc_length != c2->uc_length) return false;
-
-	/* We arrive here for less than 50% of the cases for "en" and
-	 * less then 20% of the cases for "ru", and, in practice, the
-	 * two strings are always equal, because there is almost never
-	 * a hash collision that would lead to a miscompare, because
-	 * we are hashing, at most, a few dozen connectors into a
-	 * 16-bit hash space (65536 slots).
-	 */
-	const char *uc1 = &c1->string[c1->uc_start];
-	const char *uc2 = &c2->string[c2->uc_start];
-	if (0 == strncmp(uc1, uc2, c1->uc_length)) return true;
-
-	return false;
+	return (connector_uc_num(c1) == connector_uc_num(c2));
 }
 
 static Match_node **get_match_table_entry(unsigned int size, Match_node **t,
@@ -217,7 +183,7 @@ static Match_node **get_match_table_entry(unsigned int size, Match_node **t,
 {
 	unsigned int h, s;
 
-	s = h = connector_hash(c) & (size-1);
+	s = h = connector_uc_hash(c) & (size-1);
 
 	if (dir == 1) {
 		while (NULL != t[h])
@@ -255,12 +221,13 @@ static Match_node **get_match_table_entry(unsigned int size, Match_node **t,
  * dir =  1, we're putting this into a right table.
  * dir = -1, we're putting this into a left table.
  */
-static void put_into_match_table(unsigned int size, Match_node ** t,
-                                 Disjunct * d, Connector * c, int dir )
+static void put_into_match_table(Sentence sent, unsigned int size,
+                                 Match_node ** t, Disjunct * d,
+                                 Connector * c, int dir)
 {
 	Match_node *m, **xl;
 
-	m = (Match_node *) xalloc (sizeof(Match_node));
+	m = pool_alloc(sent->fm_Match_node);
 	m->next = NULL;
 	m->d = d;
 
@@ -279,7 +246,7 @@ fast_matcher_t* alloc_fast_matcher(const Sentence sent)
 {
 	unsigned int size;
 	size_t w;
-	int len;
+	size_t len;
 	Match_node ** t;
 	Disjunct * d;
 	fast_matcher_t *ctxt;
@@ -296,9 +263,22 @@ fast_matcher_t* alloc_fast_matcher(const Sentence sent)
 	ctxt->match_list = xalloc(ctxt->match_list_size * sizeof(*ctxt->match_list));
 	ctxt->match_list_end = 0;
 
+	if (NULL != sent->fm_Match_node)
+	{
+		pool_reuse(sent->fm_Match_node);
+	}
+	else
+	{
+		sent->fm_Match_node =
+			pool_new(__func__, "Match_node",
+			         /*num_elements*/2048, sizeof(Match_node),
+			         /*zero_out*/false, /*align*/true, /*exact*/false);
+	}
+
 	for (w=0; w<sent->length; w++)
 	{
 		len = left_disjunct_list_length(sent->word[w].d);
+		len = MIN(sent->dict->contable.num_con, len);
 		size = next_power_of_two_up(len);
 		ctxt->l_table_size[w] = size;
 		t = ctxt->l_table[w] = (Match_node **) xalloc(size * sizeof(Match_node *));
@@ -308,11 +288,13 @@ fast_matcher_t* alloc_fast_matcher(const Sentence sent)
 		{
 			if (d->left != NULL)
 			{
-				put_into_match_table(size, t, d, d->left, -1);
+				//printf("%s %d\n", connector_string(d->left), d->left->length_limit);
+				put_into_match_table(sent, size, t, d, d->left, -1);
 			}
 		}
 
 		len = right_disjunct_list_length(sent->word[w].d);
+		len = MIN(sent->dict->contable.num_con, len);
 		size = next_power_of_two_up(len);
 		ctxt->r_table_size[w] = size;
 		t = ctxt->r_table[w] = (Match_node **) xalloc(size * sizeof(Match_node *));
@@ -322,7 +304,8 @@ fast_matcher_t* alloc_fast_matcher(const Sentence sent)
 		{
 			if (d->right != NULL)
 			{
-				put_into_match_table(size, t, d, d->right, 1);
+				//printf("%s %d\n", connector_string(d->right), d->right->length_limit);
+				put_into_match_table(sent, size, t, d, d->right, 1);
 			}
 		}
 	}
@@ -367,7 +350,7 @@ static void match_stats(Connector *c1, Connector *c2)
 
 #ifdef DEBUG
 #undef N
-#define N(c) (c?c->string:"")
+#define N(c) (c?connector_string(c):"")
 
 /**
  * Print the match list, including connector match indications.
@@ -375,7 +358,7 @@ static void match_stats(Connector *c1, Connector *c2)
  * Output format:
  * MATCH_NODE list_id:  lw>lc   [=]   left<w>right   [=]    rc<rw
  *
- * Regretfully this version doesn't indicate which match shortcut have been
+ * Regretfully this version doesn't indicate which match shortcut has been
  * used, and which nodes are from mr or ml or both (the full print version
  * clutters the source code very much, as it needs to be inserted in plenty
  * of places.)
@@ -391,7 +374,7 @@ static void print_match_list(fast_matcher_t *ctxt, int id, size_t mlb, int w,
 	{
 		Disjunct *d = *m;
 
-		printf("MATCH_NODE %5d: %02d>%-9s %c %9s<%02d>%-9s %c %9s<%02d\n",
+		prt_error("MATCH_NODE %5d: %02d>%-9s %c %9s<%02d>%-9s %c %9s<%02d\n",
 		       id, lw , N(lc), d->match_left ? '=': ' ',
 		       N(d->left), w, N(d->right),
 		       d->match_right? '=' : ' ', N(rc), rw);
@@ -401,59 +384,9 @@ static void print_match_list(fast_matcher_t *ctxt, int id, size_t mlb, int w,
 #define print_match_list(...)
 #endif
 
-/**
- * Compare only the lower-case parts of two connectors. When this
- * function is called, it is assumed that the upper-case parts are
- * equal, and thus do not need to be checked again.
- *
- * We know that the uc parts of the connectors are the same,
- * because we fetch the matching lists according to the uc part or the
- * connectors to be matched. So the uc parts are not checked here. The
- * head/dependent indicators are in the caller function, and only when
- * connectors match here, to save CPU when the connectors don't match
- * otherwise. This is because h/d mismatch is rare.
- * FIXME: Use connector enumeration.
- */
-static bool match_lower_case(Connector *c1, Connector *c2)
-{
-	match_stats(c1, c2);
-
-	/* If the connectors are identical, they match. */
-	if (string_set_cmp(c1->string, c2->string)) return true;
-
-	/* If any of the connectors doesn't have a lc part, they match */
-	if ((0 == c2->lc_start) || (0 == c1->lc_start)) return true;
-
-	/* Compare the lc parts according to the connector matching rules. */
-	const char *a = &c1->string[c1->lc_start];
-	const char *b = &c2->string[c2->lc_start];
-	do
-	{
-		if (*a != *b && (*a != '*') && (*b != '*')) return false;
-		a++;
-		b++;
-	} while (*a != '\0' && *b != '\0');
-
-	return true;
-}
-
-/**
- * Return false if the connectors cannot match due to identical
- * head/dependent parts. Else return true.
- */
-static bool match_hd(Connector *c1, Connector *c2)
-{
-	if ((1 == c1->uc_start) && (1 == c2->uc_start) &&
-	    (c1->string[0] == c2->string[0]))
-	{
-		return false;
-	}
-	return true;
-}
-
 typedef struct
 {
-	const char *string;
+	const condesc_t *desc;
 	bool match;
 } match_cache;
 
@@ -470,20 +403,23 @@ static bool do_match_with_cache(Connector *a, Connector *b, match_cache *c_con)
 	/* The following uses a string-set compare - string_set_cmp() cannot
 	 * be used here because c_con->string may be NULL. */
 	match_stats(c_con->string == a->string ? NULL : a, NULL);
-	if (c_con->string == a->string)
+	UNREACHABLE(connector_desc(a) == NULL); // clang static analyzer suppression.
+	if (c_con->desc == connector_desc(a))
 	{
-		/* The match_cache string field is initialized to NULL, and this is
-		 * enough for not using uninitialized c_con->match because the
-		 * connector desc filed cannot be NULL, as it actually fetched a
-		 * non-empty match list. */
+		/* The match_cache desc field is initialized to NULL, and this is
+		 * enough because the connector desc filed cannot be NULL, as it
+		 * actually fetched a non-empty match list. */
 		PRAGMA_MAYBE_UNINITIALIZED
 		return c_con->match;
 		PRAGMA_END
 	}
 
-	/* No cache exists. Check if the connectors match and cache the result. */
-	c_con->match = match_lower_case(a, b) && match_hd(a, b);
-	c_con->string = a->string;
+	/* No cache match. Check if the connectors match and cache the result.
+	 * We know that the uc parts of the connectors are the same, because
+	 * we fetch the matching lists according to the uc part or the
+	 * connectors to be matched. So the uc parts are not checked here. */
+	c_con->match = lc_easy_match(connector_desc(a), connector_desc(b));
+	c_con->desc = connector_desc(a);
 
 	return c_con->match;
 }
@@ -602,7 +538,7 @@ form_match_list(fast_matcher_t *ctxt, int w,
 	mr_end = mx;
 
 	/* Construct the list of things that could match the left. */
-	mc.string = NULL;
+	mc.desc = NULL;
 	gc.gword = NULL;
 	for (mx = ml; mx != NULL; mx = mx->next)
 	{
@@ -625,7 +561,7 @@ form_match_list(fast_matcher_t *ctxt, int w,
 	 * if we are going to skip this element here because its match_left
 	 * is true, since then it means it is already included in the match
 	 * list. */
-	mc.string = NULL;
+	mc.desc = NULL;
 	gc.gword = NULL;
 	for (mx = mr; mx != mr_end; mx = mx->next)
 	{
