@@ -12,7 +12,6 @@
 /*************************************************************************/
 
 #include <limits.h>
-
 #include "api-structures.h"
 #include "count.h"
 #include "dict-common/dict-common.h"   // For Dictionary_s
@@ -33,7 +32,7 @@
 
 static Linkage linkage_array_new(int num_to_alloc)
 {
-	Linkage lkgs = (Linkage) exalloc(num_to_alloc * sizeof(struct Linkage_s));
+	Linkage lkgs = (Linkage) malloc(num_to_alloc * sizeof(struct Linkage_s));
 	memset(lkgs, 0, num_to_alloc * sizeof(struct Linkage_s));
 	return lkgs;
 }
@@ -123,9 +122,9 @@ static void process_linkages(Sentence sent, extractor_t* pex,
 	sent->num_valid_linkages = 0;
 	size_t N_invalid_morphism = 0;
 
-	size_t itry = 0;
+	int itry = 0;
 	size_t in = 0;
-	size_t maxtries;
+	int maxtries;
 
 	/* In the case of overflow, which will happen for some long
 	 * sentences, but is particularly common for the amy/ady random
@@ -209,7 +208,7 @@ static void process_linkages(Sentence sent, extractor_t* pex,
 	 * So just pretend that it's shorter than it is */
 	sent->num_linkages_alloced = sent->num_valid_linkages;
 
-	lgdebug(D_PARSE, "Info: sane_morphism(): %zu of %zu linkages had "
+	lgdebug(D_PARSE, "Info: sane_morphism(): %zu of %d linkages had "
 	        "invalid morphology construction\n", N_invalid_morphism,
 	        itry + (itry != maxtries));
 }
@@ -247,6 +246,66 @@ static void sort_linkages(Sentence sent, Parse_Options opts)
 	print_time(opts, "Sorted all linkages");
 }
 
+#define SHORTEST_SENTENCE_TO_PACK 9
+
+/**
+ * Pack all disjunct and connectors into one big memory block.
+ * This facilitate a better memory caching for long sentences
+ * (a performance gain of a few percents).
+ *
+ * The current Connector struct size is 32 bit, and future ones may be
+ * smaller, but still with a power-of-2 size.
+ * The idea is to put an integral number of connectors in each cache line
+ * (assumed to be >= Connector struct size, e.g. 64 bytes),
+ * so one connector will not need 2 cache lines.
+ *
+ * The allocated memory includes 3 sections , in that order:
+ * 1. A block for disjuncts, when it start is not aligned (the disjunct size
+ * is currently 56 bytes and cannot be reduced much).
+ * 2. A small alignment gap, that ends in a 64-byte boundary.
+ * 3. A block of connectors, which is so aligned to 64-byte boundary.
+ *
+ * FIXME: 1. Find the "best" value for SHORTEST_SENTENCE_TO_PACK.
+ * 2. Maybe this check should be done in too stages, the second one
+ * will use number of disjunct and connector thresholds.
+ */
+static void pack_sentence(Sentence sent)
+{
+	int dcnt = 0;
+	int ccnt = 0;
+
+	if (sent->length < SHORTEST_SENTENCE_TO_PACK) return;
+	for (size_t w = 0; w < sent->length; w++)
+	{
+		Disjunct *d;
+
+		for (d = sent->word[w].d; NULL != d; d = d->next)
+		{
+			dcnt++;
+			for (Connector *c = d->right; c!=NULL; c = c->next) ccnt++;
+			for (Connector *c = d->left; c != NULL; c = c->next) ccnt++;
+		}
+	}
+
+#define CONN_ALIGNMENT sizeof(Connector)
+	size_t dsize = dcnt * sizeof(Disjunct);
+	dsize = ALIGN(dsize, CONN_ALIGNMENT); /* Align connector block. */
+	size_t csize = ccnt * sizeof(Connector);
+	void *memblock = malloc(dsize + csize);
+	Disjunct *dblock = memblock;
+	Connector *cblock = (Connector *)((char *)memblock + dsize);
+	sent->disjuncts_connectors_memblock = memblock;
+
+	for (size_t i = 0; i < sent->length; i++)
+	{
+		Disjunct *word_disjuncts = sent->word[i].d;
+
+		sent->word[i].d = pack_disjuncts_dup(sent->word[i].d, &dblock, &cblock);
+		free_disjuncts(word_disjuncts);
+	}
+}
+
+
 /**
  * classic_parse() -- parse the given sentence.
  * Perform parsing, using the original link-grammar parsing algorithm
@@ -275,7 +334,7 @@ static void sort_linkages(Sentence sent, Parse_Options opts)
 void classic_parse(Sentence sent, Parse_Options opts)
 {
 	fast_matcher_t * mchxt = NULL;
-	count_context_t * ctxt;
+	count_context_t * ctxt = NULL;
 	bool pp_and_power_prune_done = false;
 	Disjunct **disjuncts_copy = NULL;
 	bool is_null_count_0 = (0 == opts->min_null_count);
@@ -284,7 +343,6 @@ void classic_parse(Sentence sent, Parse_Options opts)
 	/* Build lists of disjuncts */
 	prepare_to_parse(sent, opts);
 	if (resources_exhausted(opts->resources)) return;
-	ctxt = alloc_count_context(sent->length);
 
 	if (is_null_count_0 && (0 < max_null_count))
 	{
@@ -311,11 +369,9 @@ void classic_parse(Sentence sent, Parse_Options opts)
 				 * parsed with null_count==0. Restore the save disjuncts. */
 				if (NULL != disjuncts_copy)
 				{
+					free_sentence_disjuncts(sent);
 					for (size_t i = 0; i < sent->length; i++)
-					{
-						free_disjuncts(sent->word[i].d);
 						sent->word[i].d = disjuncts_copy[i];
-					}
 					disjuncts_copy = NULL;
 				}
 			}
@@ -323,7 +379,10 @@ void classic_parse(Sentence sent, Parse_Options opts)
 			if (is_null_count_0) opts->min_null_count = 0;
 			if (resources_exhausted(opts->resources)) break;
 
-			free_fast_matcher(mchxt);
+			free_count_context(ctxt, sent);
+			free_fast_matcher(sent, mchxt);
+			pack_sentence(sent);
+			ctxt = alloc_count_context(sent);
 			mchxt = alloc_fast_matcher(sent);
 			print_time(opts, "Initialized fast matcher");
 		}
@@ -368,6 +427,6 @@ void classic_parse(Sentence sent, Parse_Options opts)
 		for (size_t i = 0; i < sent->length; i++)
 			free_disjuncts(disjuncts_copy[i]);
 	}
-	free_count_context(ctxt);
-	free_fast_matcher(mchxt);
+	free_count_context(ctxt, sent);
+	free_fast_matcher(sent, mchxt);
 }
