@@ -443,7 +443,7 @@ static bool possible_connection(prune_context *pc,
 	 * at least one intervening connector (i.e. cannot have both
 	 * lc->next and rc->next being null).  But we only enforce this
 	 * when we think its still possible to have a complete parse,
-	 * i.e. before well allow null-linked words.
+	 * i.e. before we allow null-linked words.
 	 */
 	else
 	if (!pc->null_links &&
@@ -486,7 +486,7 @@ right_table_search(prune_context *pc, int w, Connector *c,
 
 /**
  * This returns TRUE if the right table of word w contains
- * a connector that can match to c.  shallows tells if c is shallow
+ * a connector that can match to c.  shallow tells if c is shallow
  */
 static bool
 left_table_search(prune_context *pc, int w, Connector *c,
@@ -1010,18 +1010,19 @@ static int pp_prune(Sentence sent, Parse_Options opts)
 			{
 				if (!d->marked) continue;
 				deleteme = false;
-				for (dir = 0; dir < 2; dir++)
+				for (i = 0; i < knowledge->n_contains_one_rules; i++)
 				{
-					Connector *c;
-					for (c = ((dir) ? (d->left) : (d->right)); c != NULL; c = c->next)
-					{
-						for (i = 0; i < knowledge->n_contains_one_rules; i++)
-						{
-							pp_rule* rule = &knowledge->contains_one_rules[i]; /* the ith rule */
-							const char * selector = rule->selector;  /* selector string for this rule */
-							pp_linkset * link_set = rule->link_set;  /* the set of criterion links */
+					pp_rule* rule = &knowledge->contains_one_rules[i]; /* the ith rule */
+					const char * selector = rule->selector;  /* selector string for this rule */
+					pp_linkset * link_set = rule->link_set;  /* the set of criterion links */
 
-							if (strchr(selector, '*') != NULL) continue;  /* If it has a * forget it */
+					if (rule->selector_has_wildcard) continue;  /* If it has a * forget it */
+
+					for (dir = 0; dir < 2; dir++)
+					{
+						Connector *c;
+						for (c = ((dir) ? (d->left) : (d->right)); c != NULL; c = c->next)
+						{
 
 							if (!post_process_match(selector, connector_string(c))) continue;
 
