@@ -26,6 +26,7 @@
 #include "pp_linkset.h"
 #include "string-set.h"
 #include "dict-common/file-utils.h" // for dictopen
+#include "utilities.h"              // for free()
 
 #define D_PPK 10                       /* verbosity level for this file */
 #define PP_MAX_UNIQUE_LINK_NAMES 1024  /* just needs to be approximate */
@@ -307,6 +308,7 @@ static bool read_contains_rules(pp_knowledge *k, const char *label,
       }
 
       (*rules)[r].selector = string_set_add(tokens[0], k->string_set);
+      (*rules)[r].selector_has_wildcard = (strchr(tokens[0], '*') != NULL);
 
       /* read link set */
       tokens = pp_lexer_get_next_group_of_tokens_of_label(k->lt, &n_tokens);
