@@ -463,7 +463,7 @@ void SATEncoder::generate_satisfaction_for_expression(int w, int& dfs_position,
 
 Exp* SATEncoder::join_alternatives(int w)
 {
-  // join all alternatives using and OR_type node
+  // join all alternatives using an OR_type node
   Exp* exp;
   E_list* or_list = NULL;;
   for (X_node* x = _sent->word[w].x; x != NULL; x = x->next) {
@@ -1361,6 +1361,9 @@ void SATEncoder::pp_prune()
   if (_sent->dict->base_knowledge == NULL)
     return;
 
+  if (!_opts->perform_pp_prune)
+    return;
+
   pp_knowledge * knowledge;
   knowledge = _sent->dict->base_knowledge;
 
@@ -1474,11 +1477,6 @@ Linkage SATEncoder::get_next_linkage()
       linkage = create_linkage();
       sane = sane_linkage_morphism(_sent, linkage, _opts);
       if (!sane) {
-          /* We cannot elegantly add this linkage to sent->linkges[] -
-           * to be freed in sentence_delete(), since insane linkages
-           * must be there with index > num_linkages_post_processed - so
-           * they remain hidden, but num_linkages_post_processed is an
-           * arbitrary number.  So we must free it here. */
           free_linkage_connectors_and_disjuncts(linkage);
           free_linkage(linkage);
           free(linkage);
@@ -1498,18 +1496,9 @@ Linkage SATEncoder::get_next_linkage()
 
   assert(linkage, "No linkage");
 
-  /*
-   * We cannot expand the linkage array on demand, since the API uses
-   * linkage pointers, and they would become invalid if realloc()
-   * changes the address of the memory block. */
-#if 0
-  // Expand the linkage array.
-  int index = _sent->num_linkages_alloced;
-  _sent->num_linkages_alloced++;
-  size_t nbytes = _sent->num_linkages_alloced * sizeof(struct Linkage_s);
-  _sent->lnkages = (Linkage) realloc(_sent->lnkages, nbytes);
-#endif /* 0 */
-
+  /* We cannot expand the linkage array on demand, since the API uses
+   * linkage pointers, and they would become invalid if realloc() changes
+   * the address of the memory block. So it is allocated in advance. */
   if (NULL == _sent->lnkages)
   {
     _sent->num_linkages_alloced = _opts->linkage_limit;
@@ -1530,18 +1519,20 @@ Linkage SATEncoder::get_next_linkage()
     lkg[1].lifo.N_violations = 0;
 
   // Perform the rest of the post-processing
-  do_post_process(_sent->postprocessor, lkg, false);
-  if (NULL != _sent->postprocessor->violation) {
-    lkg->lifo.N_violations++;
-    lkg->lifo.pp_violation_msg = _sent->postprocessor->violation;
-    lgdebug(+D_SAT, "Postprocessing error: %s\n", lkg->lifo.pp_violation_msg);
-  } else {
-    // XXX We cannot maintain num_valid_linkages, because it starts from
-    // a high number. If we start it from 0, then on value 1 link-parser
-    // would report "Unique linkage".
-    //_sent->num_valid_linkages++;
+  if (NULL != _sent->postprocessor) {
+    do_post_process(_sent->postprocessor, lkg, false);
+    if (NULL != _sent->postprocessor->violation) {
+      lkg->lifo.N_violations++;
+      lkg->lifo.pp_violation_msg = _sent->postprocessor->violation;
+      lgdebug(+D_SAT, "Postprocessing error: %s\n", lkg->lifo.pp_violation_msg);
+    } else {
+      // XXX We cannot maintain num_valid_linkages, because it starts from
+      // a high number. If we start it from 0, then on value 1 link-parser
+      // would report "Unique linkage".
+      //_sent->num_valid_linkages++;
+    }
+    post_process_free_data(&_sent->postprocessor->pp_data);
   }
-  post_process_free_data(&_sent->postprocessor->pp_data);
   linkage_score(lkg, _opts);
 
   // if (NULL == _sent->postprocessor->violation && verbosity > 1)
@@ -1864,13 +1855,10 @@ bool SATEncoderConjunctionFreeSentences::sat_extract_links(Linkage lkg)
 /**
  * Main entry point into the SAT parser.
  * A note about panic mode:
- * The current version of Minisat we use doesn't support timeouts.
- * (Minisat >= 2.2 supports timeout.) In addition, currently parsing
- * with null words is not supported here.
+ * - The MiniSAT support for timeout is not yet used (FIXME).
+ * - Parsing with null links is not supported (FIXME).
  * So nothing particularly useful happens in a panic mode, and it is
  * left for the user to disable it.
- * Observation: Apparently the panic options somehow may cause an
- * immediate failure to find a solution (not checked why).
  */
 extern "C" int sat_parse(Sentence sent, Parse_Options  opts)
 {
@@ -1892,7 +1880,6 @@ extern "C" int sat_parse(Sentence sent, Parse_Options  opts)
     delete encoder;
   }
 
-  // Prepare for parsing - extracted for "preparation.c"
   encoder = new SATEncoderConjunctionFreeSentences(sent, opts);
   sent->hook = encoder;
   encoder->encode();
@@ -1922,6 +1909,7 @@ extern "C" int sat_parse(Sentence sent, Parse_Options  opts)
   if (lkg == NULL || k == linkage_limit) {
     // We don't have a valid linkages among the first linkage_limit ones
     sent->num_valid_linkages = 0;
+    sent->num_linkages_found = k;
     sent->num_linkages_post_processed = k;
     if (opts->max_null_count > 0) {
       // The sat solver doesn't support (yet) parsing with nulls.
@@ -1931,12 +1919,13 @@ extern "C" int sat_parse(Sentence sent, Parse_Options  opts)
                 "parsing with null links.\n");
     }
   } else {
-    /* We found a valid linkage.
-     * XXX However, the following setting is wrong, as we actually don't
-     * know yet the number of linkages...
-     * If we don't return some large number here, then the
-     * Command-line client will fail to print all of the possible
-     * linkages. Work around this by lying... */
+    /* We found a valid linkage. However, we actually don't know yet the
+     * number of linkages, and if we set them too low, the command-line
+     * client will fail to display all of the possible linkages.  Work
+     * around this by lying... return the maximum number of linkages we
+     * are going to produce.
+     * A NULL linkage will be returned by linkage_create() after the last
+     * linkage is produced to signify that there are no more linkages. */
     sent->num_valid_linkages = linkage_limit;
     sent->num_linkages_post_processed = linkage_limit;
   }
@@ -1955,8 +1944,8 @@ extern "C" Linkage sat_create_linkage(LinkageIdx k, Sentence sent, Parse_Options
   SATEncoder* encoder = (SATEncoder*) sent->hook;
   if (!encoder) return NULL;
 
-                                                 // linkage index k is:
-  if (k >= encoder->_sent->num_valid_linkages)   // > allocated memory
+                                               // linkage index k is:
+  if (k >= opts->linkage_limit)                  // > allocated memory
     return NULL;
   if(k > encoder->_next_linkage_index)           // skips unproduced linkages
   {
