@@ -1,6 +1,7 @@
 #ifndef __VARIABLES_HPP__
 #define __VARIABLES_HPP__
 
+#include <ctype.h>
 #include <vector>
 #include <map>
 #include <iostream>
@@ -38,6 +39,11 @@ static char* construct_link_label(const char* connector1, const char* connector2
   char* result = (char*)xalloc((std::max(strlen(connector1), strlen(connector2)) + 1)*
                                sizeof(char));
   char* presult = result;
+
+  /* Skip head-dependent indicator in each connector. */
+  if (islower(*connector1)) connector1++;
+  if (islower(*connector2)) connector2++;
+
   while (*connector1 != '\0' && *connector2 != '\0') {
     if (*connector1 == '*')
       *presult++ = *connector2;
@@ -68,7 +74,9 @@ public:
     ,_linked_min_variable_map(sent->length, -1)
     ,_linked_max_variable_map(sent->length, -1)
     ,_thin_link_variable_map(sent->length, -1)
-    ,_link_top_cw_variable_map(sent->length)
+#if 0
+      ,_link_top_cw_variable_map(sent->length)
+#endif
     ,_link_cw_variable_map(sent->length)
     ,_guiding(new CostDistanceGuiding(sent))
     ,_var(0)
@@ -413,6 +421,7 @@ public:
    *           link_top_cw((wi, pi), wj)
    */
 
+#if 0
   // Returns indices of all link_top_cw variables
   const std::vector<int>& link_top_cw_variables() const {
     return _link_top_cw_variables_indices;
@@ -431,7 +440,6 @@ public:
     const char* connector;
   };
 
-#if 0
   // Returns additional info about the given link_top_cw variable
   const LinkTopCWVar* link_top_cw_variable(int var) const {
     return _link_top_cw_variables[var];
@@ -537,6 +545,7 @@ private:
   // What is the number of the thin_link(i, j) variable?
   MatrixUpperTriangle<int> _thin_link_variable_map;
 
+#if 0
   /*
    * Information about the link_top_cw(w, wj, pj) variables
    */
@@ -549,6 +558,7 @@ private:
 
   // Additional info about the link_top_cw(wi, wj, pj) variable with the given number
   std::vector<LinkTopCWVar*> _link_top_cw_variables;
+#endif
 
 #if 0
   // Set this additional info
