@@ -12,6 +12,9 @@
 /*************************************************************************/
 
 #include <ctype.h>
+#ifdef HAVE_POSIX_MEMALIGN
+#include <errno.h>
+#endif
 #include <limits.h>
 #ifdef _WIN32
 #define _CRT_RAND_S
@@ -160,9 +163,9 @@ size_t lg_mbrtowc(wchar_t *pwc, const char *s, size_t n, mbstate_t *ps)
  * Emulate rand_r() using rand_s() in a way that is enough for our needs.
  * Windows doesn't have rand_r(), and its rand_s() is different: It
  * returns an error indication and not the random number like rand_r().
- * The value its returns is through its argument.
+ * The value it returns is through its argument.
  *
- * Note that "#define _CRT_RAND_S" is needed before "#include <stdlib.h>"
+ * Note that "#define _CRT_RAND_S" is needed before "#include <stdlib.h>".
  */
 int rand_r(unsigned int *s)
 {
@@ -281,6 +284,21 @@ void upcase_utf8_str(char *to, const char * from, size_t usize, locale_t locale)
 }
 #endif
 
+#ifdef NO_ALIGNED_MALLOC
+#if __GNUC__
+#warning No aligned alloc found (using malloc() instead).
+#endif
+#endif /* NO_ALIGNED_MALLOC */
+
+#ifdef HAVE_POSIX_MEMALIGN
+void *aligned_alloc(size_t alignment, size_t size)
+{
+	void *ptr;
+	errno = posix_memalign(&ptr, alignment, size);
+	return ptr;
+}
+#endif /* HAVE_POSIX_MEMALIGN */
+
 /* ============================================================= */
 /* Memory alloc routines below. These routines attempt to keep
  * track of how much space is getting used during a parse.
@@ -465,6 +483,16 @@ void dyn_strcat(dyn_str* ds, const char *str)
 	ds->end += l;
 }
 
+/// Trim away trailing whitespace.
+void dyn_trimback(dyn_str* ds)
+{
+	size_t tail = ds->end;
+	while (0 < tail && ' ' == ds->str[--tail]) {}
+
+	ds->end = ++tail;
+	ds->str[tail] = 0x0;
+}
+
 const char * dyn_str_value(dyn_str* s)
 {
 	return s->str;
