@@ -238,9 +238,11 @@ static const char *switch_value_string(const Switch *as)
 			snprintf(buf, sizeof(buf), "%d", ival(*as));
 			break;
 		case String:
+#if 0 /* It seems it is not a good idea. */
 			if ((NULL == *(char **)as->ptr) || ('\0' == **(char **)as->ptr))
-				strcpy(buf, "    (not set)");
+				strcpy(buf, "(not set)");
 			else
+#endif
 				snprintf(buf, sizeof(buf), "%s", *(char **)as->ptr);
 			break;
 		case Cmd:
@@ -552,13 +554,13 @@ static int help_cmd(const Switch *uc, int n)
 	printf("Special commands always begin with \"!\".  Command and variable names\n");
 	printf("can be abbreviated.  Here is a list of the commands:\n\n");
 
-	printf(" !help command   Show a detailed help for the given command\n");
+	printf(" !help command   Show a detailed help for the given command.\n");
 	for (int i = 0; uc[i].string != NULL; i++)
 	{
 		if (Cmd != uc[i].param_type) continue;
 		if (UNDOC[0] == uc[i].description[0]) continue;
 		printf(" !%-14s ", uc[i].string);
-		printf("%s\n", uc[i].description);
+		printf("%s.\n", uc[i].description);
 	}
 
 	printf("\n");
@@ -660,6 +662,8 @@ static int x_issue_special_command(char * line, Command_Options *copts, Dictiona
 		if (((Bool == as[i].param_type) || (Cmd == as[i].param_type)) &&
 		    strncasecmp(s, as[i].string, strlen(s)) == 0)
 		{
+			if ((UNDOC[0] == as[i].description[0]) &&
+			    (strlen(as[i].string) != strlen(s))) continue;
 			count++;
 			j = i;
 		}
@@ -667,7 +671,7 @@ static int x_issue_special_command(char * line, Command_Options *copts, Dictiona
 
 	if (count > 1)
 	{
-		printf("Ambiguous command.  Type \"!help\" or \"!variables\"\n");
+		prt_error("Ambiguous command \"%s\".  Type \"!help\" or \"!variables\"\n", s);
 		return -1;
 	}
 	else if (count == 1)
@@ -743,6 +747,8 @@ static int x_issue_special_command(char * line, Command_Options *copts, Dictiona
 			if (Cmd == as[i].param_type) continue;
 			if (strncasecmp(x, as[i].string, strlen(x)) == 0)
 			{
+				if ((UNDOC[0] == as[i].description[0]) &&
+				    (strlen(as[i].string) != strlen(x))) continue;
 				j = i;
 				count ++;
 			}
@@ -750,13 +756,13 @@ static int x_issue_special_command(char * line, Command_Options *copts, Dictiona
 
 		if (j<0)
 		{
-			printf("There is no user variable called \"%s\".\n", x);
+			prt_error("Error: There is no user variable called \"%s\".\n", x);
 			return -1;
 		}
 
 		if (count > 1)
 		{
-			printf("Ambiguous variable.  Type \"!help\" or \"!variables\"\n");
+			prt_error("Error: Ambiguous variable \"%s\".  Type \"!help\" or \"!variables\"\n", x);
 			return -1;
 		}
 
@@ -770,7 +776,7 @@ static int x_issue_special_command(char * line, Command_Options *copts, Dictiona
 
 			if (val < 0)
 			{
-				printf("Invalid value %s for variable %s Type \"!help\" or \"!variables\"\n", y, as[j].string);
+				prt_error("Error: Invalid value %s for variable \"%s\". Type \"!help\" or \"!variables\"\n", y, as[j].string);
 				return -1;
 			}
 
@@ -783,9 +789,9 @@ static int x_issue_special_command(char * line, Command_Options *copts, Dictiona
 		{
 			char *err;
 			double val = strtod(y, &err);
-			if ('\0' != *err)
+			if (('\0' == *y) ||('\0' != *err))
 			{
-				printf("Invalid value %s for variable %s Type \"!help\" or \"!variables\"\n", y, as[j].string);
+				prt_error("Error: Invalid value %s for variable \"%s\". Type \"!help\" or \"!variables\"\n", y, as[j].string);
 				return -1;
 			}
 
@@ -816,6 +822,8 @@ static int x_issue_special_command(char * line, Command_Options *copts, Dictiona
 		if ((Bool != as[i].param_type) && (Cmd != as[i].param_type) &&
 		    strncasecmp(s, as[i].string, strlen(s)) == 0)
 		{
+			if ((UNDOC[0] == as[i].description[0]) &&
+			    (strlen(as[i].string) != strlen(s))) continue;
 			j = i;
 			count++;
 		}
@@ -823,11 +831,11 @@ static int x_issue_special_command(char * line, Command_Options *copts, Dictiona
 
 	if (0 < count)
 	{
-		printf("Variable \"%s\" requires a value.  Try \"!help\".\n", as[j].string);
+		prt_error("Error: Variable \"%s\" requires a value.  Try \"!help\".\n", as[j].string);
 		return -1;
 	}
 
-	printf("I can't interpret \"%s\" as a command.  Try \"!help\".\n", line);
+	prt_error("Error: I can't interpret \"%s\" as a command.  Try \"!help\".\n", line);
 	return -1;
 }
 
@@ -848,7 +856,9 @@ static void put_opts_in_local_vars(Command_Options* copts)
 	local.max_cost = parse_options_get_disjunct_cost(opts);
 	local.use_cluster_disjuncts = parse_options_get_use_cluster_disjuncts(opts);
 	local.use_sat_solver = parse_options_get_use_sat_parser(opts);
+#ifdef USE_VITERBI
 	local.use_viterbi = parse_options_get_use_viterbi(opts);
+#endif
 
 	local.screen_width = (int)copts->screen_width;
 	local.echo_on = copts->echo_on;
@@ -889,7 +899,9 @@ static void put_local_vars_in_opts(Command_Options* copts)
 #ifdef USE_SAT_SOLVER
 	parse_options_set_use_sat_parser(opts, local.use_sat_solver);
 #endif
+#ifdef USE_VITERBI
 	parse_options_set_use_viterbi(opts, local.use_viterbi);
+#endif
 	parse_options_set_display_morphology(opts, local.display_morphology);
 
 	copts->screen_width = (size_t)local.screen_width;
